//# Filename: fragmentShader3D.glsl
//#
//# Synopsis: This shader carries out ray-casting based 2D rendition.
//#           Three textures are necessary: 3D volume data, 3D
//#              control data and 1D transfer function.
//#
//# Copyright (C) 2011
//#       Lionis de Souza Watanabe, Wu Shin-Ting, Jose Elias Yuri Vidalon
//#
//#   This program is free software: you can redistribute it and/or modify
//#   it under the terms of the GNU Affero General Public License as
//#   published by the Free Software Foundation, either version 3 of the
//#   License, or (at your option) any later version.
//#
//#   This program is distributed in the hope that it will be useful,
//#   but WITHOUT ANY WARRANTY; without even the implied warranty of
//#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//#   GNU Affero General Public License for more details.
//#
//#   You should have received a copy of the GNU Affero General Public License
//#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
//#
//

#extension GL_ARB_texture_rectangle : enable //rectangle texture

uniform sampler3D IMAGE3D;
uniform sampler3D CONTROLVOLUME;
uniform sampler1D TRANSFERFUNCTION1D;
uniform sampler1D TRANSFERFUNCTIONCOLOR1D;
uniform vec4 scaleFactor;
uniform vec3 extents;
uniform float stepSize;
uniform float depth_slicing;
uniform bool useProbe3D;
uniform vec3 probe3DPosition;
uniform float probeRadius;
uniform vec4 cutPlanePoint; //(x,y,z,1.0)
uniform vec4 cutPlaneVec; //(x,y,z,0.0)

void main(){
	int markup;
	float scalar, depth;
	vec4 srcColor, srcColorAux, dstColor, markupColor, controlTag;
	vec3 cursor_3d;
	vec3 probe_3d;
	vec3 geomPos, cursorColor;
	vec2 index;

	index = vec2(0.0);	
	srcColor = vec4(0.0);	
	dstColor = vec4(0.0);
	srcColorAux = vec4(0.0);	
	cursorColor = vec3(0.8, 0.8, 0.8);
	geomPos = gl_TexCoord[1].xyz;

	controlTag = texture3D(CONTROLVOLUME, geomPos.xyz);
	markup = int(controlTag.x * float(INT_FORMAT_BITS));
	
	//all markups (except transparency) are considered valid voxels
	//to be included in visualization
	switch (markup){
	case TAG001://transparent
		break;
	case TAG002://painted
		dstColor = vec4(0.0, 1.0, 1.0, 1.0);
		break;
	case TAG003://point of reference
		dstColor = vec4(1.0, 0.0, 1.0, 1.0);
		break;
	case TAG004://point of interest
		dstColor = vec4(1.0, 1.0, 0.0, 1.0);
		break;
	case TAG005://line
		dstColor = vec4(1.0, 1.0, 1.0, 1.0);
		break;
	default: //if there wasn't a markup tag, begins depth processing 
		depth = controlTag.y;//get voxel's depth
		
		//1) all untagged voxels have depth=0 and must be shown.
		//2) also, voxels deeper (with greater depth) than the slicing mesh have to be shown.
		if(depth == 0.0 || depth_slicing <= depth){
			scalar = texture3D(IMAGE3D, geomPos.xyz).r;//get RAW scalar value
			//apply default 1D transfer function
			srcColor = texture1D(TRANSFERFUNCTION1D, scalar);
			//srcColor.rgb *= srcColor.a; //compute associated color of voxel
			//apply color 1D selection transfer function
			srcColorAux = texture1D(TRANSFERFUNCTIONCOLOR1D, scalar);
			//srcColorAux.rgb *= srcColorAux.a; //compute associated color of voxel
			if(useProbe3D) {								
				//probe center at texture space [0,1]
				probe_3d.xyz = probe3DPosition.xyz * scaleFactor.xyz;
				
				//Euclidean distance to determine the voxels that belong to probe
				//to be feedback as a perfect sphere, it is necessary to rescale.								
				if ( dot((probe_3d-geomPos)*extents.xyz,(probe_3d- geomPos)*extents.xyz) <= (probeRadius*probeRadius)){						 						 
					srcColor = srcColorAux;
				}																					
			} 
//					}									
			//linear interpolation of dstColor and srcColor
			//dstColor.rgb += mix(vec3(0.0), srcColor.rgb, 1.0 - dstColor.a);
			//dstColor.rgb += (1.0 - dstColor.a) * srcColor.rgb;				
			//dstColor.a += (1.0 - dstColor.a) * srcColor.a;
			dstColor.rgb = srcColor.rgb;				
		}//End If Depth
		break;
	}//End switch

	//if (dstColor.r == 0.0) 				
	//  gl_FragColor = vec4(0.5,0.5,0.5,1.0);
	//else
	  gl_FragColor = dstColor;
}





